#!/usr/bin/env python
import sys
from django.conf import settings, global_settings as default_settings
from django.core.management import execute_from_command_line
from django import setup


if not settings.configured:

    settings.configure(
        DEBUG = False,  # will be False anyway by DjangoTestRunner.
        TEMPLATE_DEBUG = True,
        DATABASES = {
            'default': {
                'ENGINE': 'django.db.backends.postgresql_psycopg2',
                'NAME': 'test',
                'USER': '',
                'PASSWORD': '',
                'HOST': 'localhost',
                'PORT': '',
            }
        },
        TEMPLATE_LOADERS = (
            'django.template.loaders.app_directories.Loader',
            'django.template.loaders.filesystem.Loader',
        ),
        INSTALLED_APPS = (
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sites',
            'django.contrib.admin',
            'django.contrib.sessions',
            'pg_fts',
            'testapp'
        ),
        ROOT_URLCONF = 'testproject.urls',
        TEST_RUNNER='django.test.runner.DiscoverRunner',
        LANGUAGE_CODE = 'en',
        MIDDLEWARE_CLASSES=default_settings.MIDDLEWARE_CLASSES
    )


def runtests():
    setup()
    argv = sys.argv[:1] + ['test', 'testapp'] + sys.argv[1:]

    execute_from_command_line(argv)


if __name__ == '__main__':
    runtests()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.test import TestCase
from pg_fts.fields import TSVectorField
from django.db import models

__all__ = ('TestChecks', )


class TestChecks(TestCase):

    def test_check_erros(self):
        class TSVectorModelClean(models.Model):
            title = models.CharField(max_length=50)
            body = models.TextField()

            tsvector = TSVectorField(
                (('title', 'A'), 'body'))

        error_clean = TSVectorModelClean._meta.get_field('tsvector')
        self.assertEqual(len(error_clean.check()), 0)

        class TSVectorModelError(models.Model):
            title = models.CharField(max_length=50)
            body = models.TextField()

            tsvector = TSVectorField(
                (('x', 'z'), 'body'))

        error = TSVectorModelError._meta.get_field('tsvector')

        self.assertEqual(len(error.check()), 2)




class TranslationDictionary(object):
    """
    TranslationDictionary

    """
    def __init__(self, dictionaries=None, default=None):
        self.dictionaries = dictionaries or {
            'pt': ('portuguese', _('Portuguese')),
            'en': ('english', _('English')),
            'es': ('spanish', _('Spanish')),
            'de': ('german', _('German')),
            'da': ('danish', _('Danish')),
            'nl': ('dutch', _('Dutch')),
            'fi': ('finnish', _('Finnish')),
            'fr': ('french', _('French')),
            'hu': ('hungarian', _('Hungarian')),
            'it': ('italian', _('Italian')),
            'nn': ('norwegian', _('Norwegian')),
            'ro': ('romanian', _('Romanian')),
            'ru': ('russian', _('Russian')),
            'sv': ('swedish', _('Swedish')),
            'tr': ('turkish', _('Turkish')),
        }

        self.default = default or ('simple', _('Simple'))

    def get_dictionary_tuple(self, language):
        return self.dictionaries.get(language.split('-')[0], self.default)

    def get_dictionary_pg(self, language):
        return self.get_dictionary_tuple(language)[0]

    def get_dictionaries(self, languages=None):
        if languages:
            return tuple(self.get_dictionary(l) for l in self.dictionaries)
        return self.dictionaries.values()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from pg_fts.migrations import (DeleteFTSTriggerOperation,
                               DeleteFTSIndexOperation)


class Migration(migrations.Migration):

    dependencies = [
        ("testapp", "0004_create_trigger"),
    ]

    operations = [
        DeleteFTSTriggerOperation(
            name='TSVectorModel',
            fts_vector='tsvector'
            ),
        DeleteFTSIndexOperation(
            name='TSVectorModel',
            fts_vector='tsvector',
            index='gin'
            ),
        migrations.DeleteModel(
            name='TSVectorModel'
        )
    ]



"""
Classes to represent the default SQL aggregate functions
"""
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db.models.fields import FloatField
from django.db.models.constants import LOOKUP_SEP
from django.db.models.sql import aggregates
from django.core import exceptions
from pg_fts.fields import TSVectorBaseField

__all__ = ('FTSRankCd', 'FTSRank', 'FTSRankDictionay', 'FTSRankCdDictionary')


class AggregateRegister(aggregates.Aggregate):
    """
    Fake aggregate to add rank to querysets
    """
    is_ordinal = False
    is_computed = True
    sql_template = ("%(function)s(%(weights)s%(field_name)s, to_tsquery('%(dictionary)s',"
                    " %(place)s)%(normalization)s)")

    def __init__(self, col, source=None, sql_function=None, **extra):
        self.col, self.source, self.sql_function = col, source, sql_function
        self.extra, self.field = extra, FloatField()
        self.params = self.extra['params']
        self.normalization = self.extra['_normalization']
        self.weights = self.extra['_weights']

    def as_sql(self, qn, connection):
        field_name = '.'.join(qn(c) for c in self.col)
        if self.normalization:
            normalization_params = ', ' + '|'.join(
                '%d' % i for i in self.normalization)
        else:
            normalization_params = ''

        if self.weights:
            weights_params = "'{" + ', '.join(
                '%.1f' % i for i in self.weights) + "}', "
        else:
            weights_params = ''

        substitutions = {
            'field_name': '.'.join(qn(c) for c in self.col),
            'function': self.sql_function,
            'place': '%s',
            'normalization': normalization_params,
            'weights': weights_params
        }
        substitutions.update(self.extra)
        return self.sql_template % substitutions, [self.params]


class RankBase(object):
    NORMALIZATION = (0, 1, 2, 4, 8, 16, 32)
    sql_function, rhs, dictionary, srt_lookup = '', '', '', ''

    def __init__(self, lookup, **extra):
        self.lookup, self.extra = lookup, extra

    def _default_alias(self):
        return '%s__%s' % (self.lookup, self.name.lower())
    default_alias = property(_default_alias)

    def add_to_query(self, query, alias, col, source, is_summary=False):
        if self.dictionary:
            # test for if is a valid transform, if not will raise error
            source.get_transform(self.dictionary)
        else:
            self.dictionary = source.dictionary

        lookup = source.get_lookup(self.srt_lookup)
        fts_query = lookup.lookup_sql % (
            '.'.join('"%s"' % c for c in col), self.dictionary, '%s')
        params = source._get_db_prep_lookup(self.srt_lookup, self.rhs)

        self.extra = {
            'params': params,
            'dictionary': self.dictionary,
            '_normalization': self.normalization,
            '_weights': self.weights
        }

        query.add_extra(
            select=None,
            select_params=None,
            where=[fts_query % ("%s")],
            params=[params],
            tables=None,
            order_by=None
        )

        aggregate = AggregateRegister(
            col,
            source=source,
            sql_function=self.sql_function,
            **self.extra)
        query.aggregates[alias] = aggregate
        # in case of related
        if col not in query.group_by:
            query.group_by.append(col)

    def _do_checks(self):
        assert not self.weights or (len(self.weights) is 4 and all(map(
            lambda x: isinstance(x, (int, float)),
            self.weights
        ))), 'weights must be of length 4 and type float or integer'

        assert not self.normalization or all(map(
            lambda x: (isinstance(x, int) and x in self.NORMALIZATION),
            self.normalization
        )), 'normalization must be in (%s)' % (
            ', '.join('%d' % i for i in self.NORMALIZATION))
        assert len(self.extra) == 1, 'to many arguments for %s' % (
            self.__class__.__name__)
        if self.srt_lookup not in TSVectorBaseField.valid_lookups:
            raise exceptions.FieldError(
                "The '%s' isn't valid Lookup for %s" % (
                    self.srt_lookup, self.__class__.__name__))


class FTSRank(RankBase):
    """
    Interface for PostgreSQL ts_rank

    Provides a interface for
        :pg_docs:`12.3.3. Ranking Search Results *ts_rank*
        <textsearch-controls.html#TEXTSEARCH-RANKING>`

    Example::

        Article.objects.annotate(rank=FTSRank(fts_index__search='Hello world',
                                 normalization=[1,2]))

    SQL equivalent:

    .. code-block:: sql

        SELECT
            ...,
            ts_rank("article"."fts_index" @@ to_tsquery('english', 'Hello & world'), 1|2) AS "rank"
        WHERE
            "article"."fts_index" @@ to_tsquery('english', 'Hello & world')

    :param fieldlookup: required

    :param normalization: iterable integer

    :param weights: iterable float

    :returns: rank

    :raises: exceptions.FieldError if lookup isn't valid

    """

    name = 'FTSRank'
    sql_function = 'ts_rank'
    dictionary = ''

    def __init__(self, **extra):
        self.normalization = extra.pop('normalization', [])
        self.weights = extra.pop('weights', [])
        params = tuple(extra.items())[0]
        self.extra = extra
        lookups, self.rhs = params[0].split(LOOKUP_SEP), params[1]
        self.srt_lookup = lookups[-1]
        self.lookup = LOOKUP_SEP.join(lookups[:-1])
        self._do_checks()


class FTSRankCd(FTSRank):
    """
    Interface for PostgreSQL ts_rank_cd

    Provides a interface for
        :pg_docs:`12.3.3. Ranking Search Results *ts_rank_cd*
        <textsearch-controls.html#TEXTSEARCH-RANKING>`


    :param fieldlookup: required


    :param normalization: list or tuple of integers PostgreSQL normalization
        values

    :returns: rank_cd

    :raises: exceptions.FieldError if lookup isn't valid

    Example::

        Article.objects.annotate(
            rank=FTSRank(fts_index__search='Hello world',
                         normalization=[1,2]))

    SQL equivalent:

    .. code-block:: sql

        SELECT
            ...,
            ts_rank_cd("article"."fts_index" @@ to_tsquery('english', 'Hello & world'), 1|2) AS "rank"
        WHERE
            "article"."fts_index" @@ to_tsquery('english', 'Hello & world')
    """

    sql_function = 'ts_rank_cd'
    name = 'FTSRankCd'


class FTSRankDictionay(FTSRank):
    """
    Interface for PostgreSQL ts_rank with **language lookup**

    Provides a interface for
        :pg_docs:`12.3.3. Ranking Search Results *rank*
        <textsearch-controls.html#TEXTSEARCH-RANKING>`

    :param fieldlookup: required

    :param normalization: list or tuple of integers PostgreSQL normalization
        values

    :returns: rank

    :raises: exceptions.FieldError if lookup isn't valid

    Example::

        Article.objects.annotate(
            rank=FTSRankDictionay(fts_index__portuguese__search='Hello world',
                                  normalization=[1,2]))

    SQL equivalent:

    .. code-block:: sql

        SELECT
            ...,
            ts_rank("article"."fts_index" @@ to_tsquery('portuguese', 'Hello & world'), 1|2) AS "rank"
        WHERE
            "article"."fts_index" @@ to_tsquery('portuguese', 'Hello & world')

    """

    def __init__(self, **extra):
        self.normalization = extra.pop('normalization', [])
        self.weights = extra.pop('weights', [])
        params = tuple(extra.items())[0]
        self.extra, self.rhs = extra, params[1]
        lookups = params[0].split(LOOKUP_SEP)
        self.dictionary, self.srt_lookup = lookups[-2:]
        self.lookup = LOOKUP_SEP.join(lookups[:-2])
        self._do_checks()


class FTSRankCdDictionary(FTSRankDictionay):
    """
    Interface for PostgreSQL ts_rank_cd with **language lookup**

    Provides a interface for
        :pg_docs:`12.3.3. Ranking Search Results *rank_cd*
        <textsearch-controls.html#TEXTSEARCH-RANKING>` with **language lookup**

    :param fieldlookup: required

    :param normalization: list or tuple of integers PostgreSQL normalization
        values

    :returns: rank_cd

    :raises: exceptions.FieldError if lookup isn't valid

    Example::

        Article.objects.annotate(
            rank=FTSRankCdDictionary(fts_index__portuguese__search='Hello world',
                                     normalization=[1,2]))

    SQL equivalent:

    .. code-block:: sql

        SELECT
            ...,
            ts_rank("article"."fts_index" @@ to_tsquery('portuguese', 'Hello & world'), 1|2) AS "rank"
        WHERE
            "article"."fts_index" @@ to_tsquery('portuguese', 'Hello & world')

    """

    sql_function = 'ts_rank_cd'
    name = 'FTSRankCdDictionary'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import pg_fts.fields
from pg_fts.migrations import UpdateVectorOperation


class Migration(migrations.Migration):
    dependencies = [
        ("testapp", "0001_initial"),
    ]

    operations = [

        migrations.CreateModel(
            name='TSVectorModel',
            fields=[
                ('id', models.AutoField(
                    serialize=False, auto_created=True,
                    verbose_name='ID', primary_key=True)),
                ('title', models.CharField(max_length=50)),
                ('dictionary', models.CharField(max_length=15, default='english', choices=[('english', 'english'), ('portuguese', 'portuguese')])),
                ('body', models.TextField()),
                ('tsvector', pg_fts.fields.TSVectorField(
                    db_index=True, editable=False,
                    dictionary='dictionary', default='',
                    fields=('title', 'body'),
                    serialize=False, null=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        UpdateVectorOperation(
            name='TSVectorModel',
            fts_vector='tsvector'
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import pg_fts.fields
from pg_fts.migrations import CreateFTSIndexOperation, CreateFTSTriggerOperation


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TSMultidicModel',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, verbose_name='ID', primary_key=True)),
                ('title', models.CharField(max_length=50)),
                ('body', models.TextField()),
                ('dictionary', models.CharField(max_length=15, default='english', choices=[('english', 'english'), ('portuguese', 'portuguese')])),
                ('tsvector', pg_fts.fields.TSVectorField(editable=False, dictionary='dictionary', default='', fields=(('title', 'A'), 'body'), serialize=False, null=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='TSQueryModel',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, verbose_name='ID', primary_key=True)),
                ('title', models.CharField(max_length=50)),
                ('body', models.TextField()),
                ('tsvector', pg_fts.fields.TSVectorField(editable=False, dictionary='english', default='', fields=('title', 'body'), serialize=False, null=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        CreateFTSIndexOperation(
            name='TSQueryModel',
            fts_vector='tsvector',
            index='gin'
        ),
        CreateFTSIndexOperation(
            name='TSMultidicModel',
            fts_vector='tsvector',
            index='gin'
        ),
        CreateFTSTriggerOperation(
            name='TSQueryModel',
            fts_vector='tsvector',
        ),
        CreateFTSTriggerOperation(
            name='TSMultidicModel',
            fts_vector='tsvector',
        ),

    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class PgFTSIntrospection(object):
    """
        Helper class the introspect the database
    """
    def get_dictionay_list(self, cursor):
        """
        introspects pg_catalog.pg_ts_config

        :returns: A list of dictionaries names installed in postgres
        """

        'SELECT cfgname FROM "pg_catalog"."pg_ts_config"'
        return [row[0] for row in cursor.fetchall()]

    def get_trigger_list(self, cursor):
        """
        introspects the database for triggers

        :returns: A list of triggers
        """

        cursor.execute("""
            SELECT DISTINCT trigger_name
            FROM information_schema.triggers
            WHERE trigger_schema NOT IN
                ('pg_catalog', 'information_schema');""")
        return [row[0] for row in cursor.fetchall()]

    def get_functions_list(self, cursor):
        """
        introspects the database for functions

        :returns: A list of functions
        """

        cursor.execute('''SELECT p.proname AS function_name
FROM   (SELECT oid, * FROM pg_proc p WHERE NOT p.proisagg) p
JOIN   pg_namespace n ON n.oid = p.pronamespace
WHERE  n.nspname = 'public'
''')
        return [row[0] for row in cursor.fetchall()]


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.db import connection
from pg_fts.introspection import PgFTSIntrospection
from django.test import (override_settings, override_system_checks,
                         TestCase, TransactionTestCase)
from django.utils import six
from django.core.management import call_command

try:
    from django.db.backends import TableInfo
    table_info = True
except:
    table_info = False

__all__ = ('FTSTestBase', 'CreateOperationTestSQL',
           'TransactionsMigrationsTest')


class FTSTestBase(TransactionTestCase):
    '''tests for FTS'''

    introspection = PgFTSIntrospection()

    def assertIndexExists(self, index, table):
        with connection.cursor() as cursor:
            self.assertIn(
                index, connection.introspection.get_indexes(cursor, table))

    def assertIndexNotExists(self, index, table):
        with connection.cursor() as cursor:
            self.assertNotIn(
                index, connection.introspection.get_indexes(cursor, table))

    def assertTriggerExists(self, trigger):
        with connection.cursor() as cursor:
            self.assertIn(trigger, self.introspection.get_trigger_list(cursor))

    def assertTriggerNotExists(self, trigger):
        with connection.cursor() as cursor:
            self.assertNotIn(trigger,
                             self.introspection.get_trigger_list(cursor))

    def assertFunctionExists(self, function):
        with connection.cursor() as cursor:
            self.assertIn(function,
                          self.introspection.get_functions_list(cursor))

    def assertFunctionNotExists(self, function):
        with connection.cursor() as cursor:
            self.assertNotIn(function,
                             self.introspection.get_functions_list(cursor))

    def assertTableExists(self, table):
        if table_info:
            table = TableInfo(name=table, type='t')
        with connection.cursor() as cursor:
            self.assertIn(table,
                          connection.introspection.get_table_list(cursor))

    def assertTableNotExists(self, table):
        if table_info:
            table = TableInfo(name=table, type='t')

        with connection.cursor() as cursor:
            self.assertNotIn(table,
                             connection.introspection.get_table_list(cursor))


class CreateOperationTestSQL(TestCase):
    # single dictionary
    @override_system_checks([])
    @override_settings(MIGRATION_MODULES={"testapp": "testapp.migrations_index"})
    def test_sql_migrate_creates_vector_field(self):
        stdout = six.StringIO()
        call_command('sqlmigrate', 'testapp', '0002', stdout=stdout)
        self.assertIn('"tsvector" tsvector null', stdout.getvalue().lower())
        self.assertIn(
            "UPDATE testapp_tsvectormodel SET tsvector = setweight(to_tsvector('english', COALESCE(title, '')), 'D') || setweight(to_tsvector('english', COALESCE(body, '')), 'D');",
            stdout.getvalue())

    @override_system_checks([])
    @override_settings(MIGRATION_MODULES={"testapp": "testapp.migrations_index"})
    def test_sql_fts_index(self):
        stdout = six.StringIO()
        call_command('sqlmigrate', 'testapp', '0003', stdout=stdout)
        self.assertIn(
            ('CREATE INDEX testapp_tsvectormodel_tsvector ON '
             'testapp_tsvectormodel USING gin(tsvector);'),
            stdout.getvalue())

    @override_system_checks([])
    @override_settings(MIGRATION_MODULES={"testapp": "testapp.migrations_index"})
    def test_sql_fts_trigger(self):
        stdout = six.StringIO()
        call_command('sqlmigrate', 'testapp', '0004', stdout=stdout)
        self.assertIn(
            """
CREATE FUNCTION testapp_tsvectormodel_tsvector_update() RETURNS TRIGGER AS $$
BEGIN
    IF TG_OP = 'INSERT' THEN
        new.tsvector = setweight(to_tsvector('english', COALESCE(NEW.title, '')), 'D') || setweight(to_tsvector('english', COALESCE(NEW.body, '')), 'D');
    END IF;
    IF TG_OP = 'UPDATE' THEN
        IF NEW.title <> OLD.title OR NEW.body <> OLD.body THEN
            new.tsvector = setweight(to_tsvector('english', COALESCE(NEW.title, '')), 'D') || setweight(to_tsvector('english', COALESCE(NEW.body, '')), 'D');
        END IF;
    END IF;
RETURN NEW;
END;
$$ LANGUAGE 'plpgsql';
CREATE TRIGGER testapp_tsvectormodel_tsvector_update BEFORE INSERT OR UPDATE ON testapp_tsvectormodel
FOR EACH ROW EXECUTE PROCEDURE testapp_tsvectormodel_tsvector_update();""",
            stdout.getvalue()
        )

    # multiple dictionaries
    @override_system_checks([])
    @override_settings(MIGRATION_MODULES={"testapp": "testapp.migrations_multidict"})
    def test_sql_migrate_creates_vector_field_multi(self):
        stdout = six.StringIO()
        call_command('sqlmigrate', 'testapp', '0002', stdout=stdout)
        self.assertIn('"tsvector" tsvector null', stdout.getvalue().lower())
        self.assertIn(
            "UPDATE testapp_tsvectormodel SET tsvector = setweight(to_tsvector(dictionary::regconfig, COALESCE(title, '')), 'D') || setweight(to_tsvector(dictionary::regconfig, COALESCE(body, '')), 'D');",
            stdout.getvalue())

    @override_system_checks([])
    @override_settings(MIGRATION_MODULES={"testapp": "testapp.migrations_multidict"})
    def test_sql_fts_index_multi(self):
        stdout = six.StringIO()

        call_command('sqlmigrate', 'testapp', '0003', stdout=stdout)
        self.assertIn(
            ('CREATE INDEX testapp_tsvectormodel_tsvector ON '
             'testapp_tsvectormodel USING gin(tsvector);'),
            stdout.getvalue())
        self.assertIn(
            ('CREATE INDEX testapp_tsvectormodel_tsvector ON '
             'testapp_tsvectormodel USING gin(tsvector);'),
            stdout.getvalue())

    @override_system_checks([])
    @override_settings(MIGRATION_MODULES={"testapp": "testapp.migrations_multidict"})
    def test_sql_fts_trigger_multi(self):
        stdout = six.StringIO()
        call_command('sqlmigrate', 'testapp', '0004', stdout=stdout)
        self.assertIn(
            """
CREATE FUNCTION testapp_tsvectormodel_tsvector_update() RETURNS TRIGGER AS $$
BEGIN
    IF TG_OP = 'INSERT' THEN
        new.tsvector = setweight(to_tsvector(NEW.dictionary::regconfig, COALESCE(NEW.title, '')), 'D') || setweight(to_tsvector(NEW.dictionary::regconfig, COALESCE(NEW.body, '')), 'D');
    END IF;
    IF TG_OP = 'UPDATE' THEN
        IF NEW.dictionary <> OLD.dictionary OR NEW.title <> OLD.title OR NEW.body <> OLD.body THEN
            new.tsvector = setweight(to_tsvector(NEW.dictionary::regconfig, COALESCE(NEW.title, '')), 'D') || setweight(to_tsvector(NEW.dictionary::regconfig, COALESCE(NEW.body, '')), 'D');
        END IF;
    END IF;
RETURN NEW;
END;
$$ LANGUAGE 'plpgsql';
CREATE TRIGGER testapp_tsvectormodel_tsvector_update BEFORE INSERT OR UPDATE ON testapp_tsvectormodel
FOR EACH ROW EXECUTE PROCEDURE testapp_tsvectormodel_tsvector_update();""",
            stdout.getvalue()
        )


class TransactionsMigrationsTest(FTSTestBase):

    # available_apps = ["testapp"]

    @override_system_checks([])
    @override_settings(MIGRATION_MODULES={"testapp": "testapp.migrations_index"})
    def test_migrate_forwards_backwards(self):
        stdout = six.StringIO()
        call_command('migrate', 'testapp', '0002', stdout=stdout)
        self.assertTableExists('testapp_tsvectormodel')
        self.assertIndexNotExists('tsvector',
                                  'testapp_tsvectormodel')
        call_command('migrate', 'testapp', '0003', stdout=stdout)
        self.assertIndexExists('tsvector', 'testapp_tsvectormodel')
        self.assertFunctionNotExists('testapp_tsvectormodel_tsvector_update')
        self.assertTriggerNotExists('testapp_tsvectormodel_tsvector_update')
        call_command('migrate', 'testapp', '0004', stdout=stdout)
        self.assertFunctionExists('testapp_tsvectormodel_tsvector_update')
        self.assertTriggerExists('testapp_tsvectormodel_tsvector_update')
        call_command('migrate', 'testapp', '0005', stdout=stdout)
        self.assertFunctionNotExists('testapp_tsvectormodel_tsvector_update')
        self.assertTriggerNotExists('testapp_tsvectormodel_tsvector_update')
        self.assertTableNotExists('testapp_tsvectormodel')
        call_command('migrate', 'testapp', '0004', stdout=stdout)
        self.assertFunctionExists('testapp_tsvectormodel_tsvector_update')
        self.assertTriggerExists('testapp_tsvectormodel_tsvector_update')
        self.assertTableExists('testapp_tsvectormodel')
        call_command('migrate', 'testapp', '0003', stdout=stdout)
        self.assertFunctionNotExists('testapp_tsvectormodel_tsvector_update')
        self.assertTriggerNotExists('testapp_tsvectormodel_tsvector_update')
        call_command('migrate', 'testapp', '0002', stdout=stdout)
        self.assertIndexNotExists('tsvector',
                                  'testapp_tsvectormodel')
        call_command('migrate', 'testapp', '0001', stdout=stdout)

    @override_system_checks([])
    @override_settings(MIGRATION_MODULES={"testapp": "testapp.migrations_index"})
    def test_migrate_forwards_backwards_multi(self):
        stdout = six.StringIO()
        call_command('migrate', 'testapp', '0002', stdout=stdout)
        self.assertTableExists('testapp_tsvectormodel')
        self.assertIndexNotExists('tsvector',
                                  'testapp_tsvectormodel')
        call_command('migrate', 'testapp', '0003', stdout=stdout)
        self.assertIndexExists('tsvector', 'testapp_tsvectormodel')
        self.assertFunctionNotExists('testapp_tsvectormodel_tsvector_update')
        self.assertTriggerNotExists('testapp_tsvectormodel_tsvector_update')
        call_command('migrate', 'testapp', '0004', stdout=stdout)
        self.assertFunctionExists('testapp_tsvectormodel_tsvector_update')
        self.assertTriggerExists('testapp_tsvectormodel_tsvector_update')
        call_command('migrate', 'testapp', '0005', stdout=stdout)
        self.assertFunctionNotExists('testapp_tsvectormodel_tsvector_update')
        self.assertTriggerNotExists('testapp_tsvectormodel_tsvector_update')
        self.assertTableNotExists('testapp_tsvectormodel')
        call_command('migrate', 'testapp', '0004', stdout=stdout)
        self.assertFunctionExists('testapp_tsvectormodel_tsvector_update')
        self.assertTriggerExists('testapp_tsvectormodel_tsvector_update')
        self.assertTableExists('testapp_tsvectormodel')
        call_command('migrate', 'testapp', '0003', stdout=stdout)
        self.assertFunctionNotExists('testapp_tsvectormodel_tsvector_update')
        self.assertTriggerNotExists('testapp_tsvectormodel_tsvector_update')
        call_command('migrate', 'testapp', '0002', stdout=stdout)
        self.assertIndexNotExists('tsvector',
                                  'testapp_tsvectormodel')
        call_command('migrate', 'testapp', '0001', stdout=stdout)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.test import TestCase
from testapp.models import TSQueryModel, Related, TSMultidicModel
from pg_fts.ranks import (FTSRank, FTSRankDictionay, FTSRankCd,
                          FTSRankCdDictionary)
from django.core import exceptions

__all__ = ('AnnotateTestCase', 'FTSRankDictionayTestCase')


class AnnotateTestCase(TestCase):

    def setUp(self):
        a = TSQueryModel.objects.create(
            title='para for os the mesmo same malucos crazy',
            body="""para for os the mesmo same malucos crazy que that tomorow
salvão save o the planeta planet"""
        )

        b = TSQueryModel.objects.create(
            title='malucos crazy como like eu me',
            body="""para for os the mesmo same malucos crazy que that tomorow
salvão save o the planeta planet"""
        )

        Related.objects.create(single=a)
        Related.objects.create(single=b)

    def test_ts_rank_search(self):
        q = TSQueryModel.objects.annotate(
            rank=FTSRank(tsvector__search='para mesmo')
        )

        self.assertIn('''WHERE ("testapp_tsquerymodel"."tsvector" @@ to_tsquery('english', para & mesmo))''',
                      str(q.query))

        self.assertIn('''ts_rank("testapp_tsquerymodel"."tsvector", to_tsquery('english', para & mesmo)) AS "rank"''',
                      str(q.query))

        self.assertEqual(
            q.order_by('-rank')[0].title, 'para for os the mesmo same malucos crazy')

        self.assertEqual(
            q.order_by('rank')[0].title, 'malucos crazy como like eu me')

    def test_ts_rank_isearch(self):
        q = TSQueryModel.objects.annotate(
            rank=FTSRank(tsvector__isearch='para mesmo'))

        self.assertIn('''WHERE ("testapp_tsquerymodel"."tsvector" @@ to_tsquery('english', para | mesmo))''',
                      str(q.query))

        self.assertIn('''ts_rank("testapp_tsquerymodel"."tsvector", to_tsquery('english', para | mesmo)) AS "rank"''',
                      str(q.query))

        self.assertEqual(
            q.order_by('-rank')[0].title, 'para for os the mesmo same malucos crazy')

        self.assertEqual(
            q.order_by('rank')[0].title, 'malucos crazy como like eu me')

    def test_ts_rank_tsquery(self):
        q = TSQueryModel.objects.annotate(
            rank=FTSRank(tsvector__tsquery='para & mesmo'))

        self.assertIn('''WHERE ("testapp_tsquerymodel"."tsvector" @@ to_tsquery('english', para & mesmo))''',
                      str(q.query))

        self.assertIn('''ts_rank("testapp_tsquerymodel"."tsvector", to_tsquery('english', para & mesmo)) AS "rank"''',
                      str(q.query))

        self.assertEqual(
            q.order_by('-rank')[0].title, 'para for os the mesmo same malucos crazy')

        self.assertEqual(
            q.order_by('rank')[0].title, 'malucos crazy como like eu me')

    def test_ts_rank_search_related(self):
        q = Related.objects.annotate(
            rank=FTSRank(single__tsvector__search='para mesmo')
        )
        self.assertEqual(len(q), 2)
        self.assertIn('''WHERE ("testapp_tsquerymodel"."tsvector" @@ to_tsquery('english', para & mesmo))''',
                      str(q.query))

        self.assertIn('''ts_rank("testapp_tsquerymodel"."tsvector", to_tsquery('english', para & mesmo)) AS "rank"''',
                      str(q.query))

        self.assertEqual(
            q.order_by('-rank')[0].single.title, 'para for os the mesmo same malucos crazy')

        self.assertEqual(
            q.order_by('rank')[0].single.title, 'malucos crazy como like eu me')

    def test_rank_dictionay_group_by_related(self):
        qn = Related.objects.annotate(
            rank=FTSRank(single__tsvector__search='para mesmo'))
        self.assertIn('"testapp_tsquerymodel"."tsvector"',
                      str(qn.query).split('GROUP BY')[-1])

    def test_normalization(self):
        qs = TSQueryModel.objects.annotate(
            rank=FTSRank(tsvector__tsquery='para & mesmo', normalization=[32, 8]))
        self.assertIn('''ts_rank("testapp_tsquerymodel"."tsvector", to_tsquery('english', para & mesmo), 32|8) AS "rank"''',
                      str(qs.query))
        self.assertEqual(len(qs), 2)

    def test_weights(self):
        qs = TSQueryModel.objects.annotate(
            rank=FTSRank(
                tsvector__tsquery='para & mesmo',
                normalization=(32, 8),
                weights=(0.1, 0.2, 0.4, 1.0)
            )
        )

        self.assertIn('''ts_rank('{0.1, 0.2, 0.4, 1.0}', "testapp_tsquerymodel"."tsvector", to_tsquery('english', para & mesmo), 32|8) AS "rank"''',
                      str(qs.query))

        self.assertEqual(len(qs), 2)

    def test_rank_assertions(self):
        with self.assertRaises(AssertionError):
            qs = TSQueryModel.objects.annotate(
                rank=FTSRank(
                    tsvector__tsquery='para & mesmo',
                    normalization=(2, 8),
                    weights=(0.1, 'asd', 0.4, 1.0)
                )
            )
            list(qs)
        with self.assertRaises(AssertionError):
            qs = TSQueryModel.objects.annotate(
                rank=FTSRank(
                    tsvector__tsquery='para & mesmo',
                    normalization=(3, 8),
                    weights=(0.1, 2, 0.4, 1.0)
                )
            )
            list(qs)

    # need to find a way to catch FieldError raised by
    # django.db.models.sql.query in add fields
    #
    def test_transform_dictionary_exception(self):
        with self.assertRaises(exceptions.FieldError) as msg:
            TSQueryModel.objects.annotate(
                rank=FTSRank(tsvector__nodict='malucos')),
        self.assertEqual(
            str(msg.exception),
            "The 'nodict' isn't valid Lookup for FTSRank")

        with self.assertRaises(exceptions.FieldError) as msg:
            TSQueryModel.objects.annotate(
                rank=FTSRank(tsvector='malucos')),
        self.assertEqual(
            str(msg.exception),
            "The 'tsvector' isn't valid Lookup for FTSRank")

    def test_ts_rank_cd_search(self):
        q = TSQueryModel.objects.annotate(
            rank=FTSRankCd(tsvector__search='para mesmo')
        )

        self.assertIn('''WHERE ("testapp_tsquerymodel"."tsvector" @@ to_tsquery('english', para & mesmo))''',
                      str(q.query))

        self.assertIn('''ts_rank_cd("testapp_tsquerymodel"."tsvector", to_tsquery('english', para & mesmo)) AS "rank"''',
                      str(q.query))

        self.assertEqual(
            q.order_by('-rank')[0].title, 'para for os the mesmo same malucos crazy')

        self.assertEqual(
            q.order_by('rank')[0].title, 'malucos crazy como like eu me')


class FTSRankDictionayTestCase(TestCase):
    '''tests for FTSRankDictionayTestCase'''

    def setUp(self):
        title = 'para for os the mesmo same malucos crazy'
        body = """para for os the mesmo same malucos crazy que that tomorow
salvão save o the planeta planet"""
        pt = TSMultidicModel.objects.create(
            title=title,
            body=body,
            dictionary='english'
        )

        en = TSMultidicModel.objects.create(
            title=title,
            body=body,
            dictionary='portuguese'
        )

        Related.objects.create(
            multiple=pt
        )

        Related.objects.create(
            multiple=en
        )

    def test_rank_dictionay_transform_search(self):
        # `para``os` are stopwords in portuguese
        qn_base_pt = TSMultidicModel.objects.filter(dictionary='portuguese')
        qn_base_en = TSMultidicModel.objects.filter(dictionary='english')
        pt = qn_base_pt.annotate(
            rank=FTSRankDictionay(tsvector__portuguese__tsquery='para & os'))

        self.assertIn(
            '''("testapp_tsmultidicmodel"."tsvector" @@ to_tsquery('portuguese', para & os))''',
            str(pt.query))

        self.assertIn(
            '''ts_rank("testapp_tsmultidicmodel"."tsvector", to_tsquery('portuguese', para & os)) AS "rank"''',
            str(pt.query))

        en = qn_base_pt.annotate(
            rank=FTSRankDictionay(tsvector__english__tsquery='para & os'))

        self.assertIn(
            '''("testapp_tsmultidicmodel"."tsvector" @@ to_tsquery('english', para & os))''',
            str(en.query))

        self.assertIn(
            '''ts_rank("testapp_tsmultidicmodel"."tsvector", to_tsquery('english', para & os)) AS "rank"''',
            str(en.query))

        qn_base_pt.annotate(
            rank=FTSRankDictionay(tsvector__portuguese__tsquery='para & os')
        )

    def test_rank_dictionay_related_multidict(self):
        qn_base_pt = Related.objects.filter(multiple__dictionary='portuguese')
        qn_base_en = Related.objects.filter(multiple__dictionary='english')
        qn_pt = qn_base_pt.annotate(rank=FTSRankDictionay(
            multiple__tsvector__portuguese__tsquery='para & os'))

        qn_en = qn_base_en.annotate(rank=FTSRankDictionay(
            multiple__tsvector__english__tsquery='para & os'))

        self.assertIn('''"testapp_tsmultidicmodel"."tsvector" @@ to_tsquery('english', para & os))''',
                      str(qn_en.query))

        self.assertIn('''ts_rank("testapp_tsmultidicmodel"."tsvector", to_tsquery('english', para & os)) AS "rank"''',
                      str(qn_en.query))

        self.assertIn('''"testapp_tsmultidicmodel"."tsvector" @@ to_tsquery('portuguese', para & os))''',
                      str(qn_pt.query))

        self.assertIn('''ts_rank("testapp_tsmultidicmodel"."tsvector", to_tsquery('portuguese', para & os)) AS "rank"''',
                      str(qn_pt.query))

        self.assertEqual(len(qn_en), 1)
        self.assertEqual(len(qn_pt), 0)

    def test_rank_dictionay_group_by_related(self):
        qn_base_pt = Related.objects.filter(multiple__dictionary='portuguese')
        qn_pt = qn_base_pt.annotate(rank=FTSRankDictionay(
            multiple__tsvector__portuguese__tsquery='para & os'))

        self.assertIn('"testapp_tsmultidicmodel"."tsvector"',
                      str(qn_pt.query).split('GROUP BY')[-1])

    def test_rank_cd_dictionary(self):
        qn_base_pt = Related.objects.filter(multiple__dictionary='portuguese')
        qn_base_en = Related.objects.filter(multiple__dictionary='english')
        qn_pt = qn_base_pt.annotate(rank=FTSRankCdDictionary(
            multiple__tsvector__portuguese__tsquery='para & os'))

        qn_en = qn_base_en.annotate(rank=FTSRankCdDictionary(
            multiple__tsvector__english__tsquery='para & os'))

        self.assertIn('''"testapp_tsmultidicmodel"."tsvector" @@ to_tsquery('english', para & os))''',
                      str(qn_en.query))

        self.assertIn('''ts_rank_cd("testapp_tsmultidicmodel"."tsvector", to_tsquery('english', para & os)) AS "rank"''',
                      str(qn_en.query))

        self.assertIn('''"testapp_tsmultidicmodel"."tsvector" @@ to_tsquery('portuguese', para & os))''',
                      str(qn_pt.query))

        self.assertIn('''ts_rank_cd("testapp_tsmultidicmodel"."tsvector", to_tsquery('portuguese', para & os)) AS "rank"''',
                      str(qn_pt.query))

    def test_transform_dictionary_exception(self):
        with self.assertRaises(exceptions.FieldError) as msg:
            TSMultidicModel.objects.annotate(
                rank=FTSRankDictionay(tsvector__nodict='malucos')),
        self.assertEqual(
            str(msg.exception),
            "The 'nodict' isn't valid Lookup for FTSRankDictionay")

    def test_transform_exception(self):
        with self.assertRaises(exceptions.FieldError) as msg:
            list(TSMultidicModel.objects.annotate(
                rank=FTSRankDictionay(tsvector__portuguese='malucos')))
        self.assertEqual(
            str(msg.exception),
            "The 'portuguese' isn't valid Lookup for FTSRankDictionay")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from pg_fts.migrations import CreateFTSIndexOperation, CreateFTSTriggerOperation


class Migration(migrations.Migration):

    dependencies = [
        ("testapp", "0002_create_model"),
    ]

    operations = [
        CreateFTSIndexOperation(
            name='TSVectorModel',
            fts_vector='tsvector',
            index='gin'
            ),
        ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.db.models import Field, Lookup, Transform
from django.db.models.lookups import RegisterLookupMixin
from django.utils import six
from django.core import checks, exceptions
from django.utils.translation import string_concat, ugettext_lazy as _
from django.db import models
import re

__all__ = ('TSVectorField', 'TSVectorBaseField', 'TSVectorTsQueryLookup',
           'TSVectorSearchLookup', 'TSVectorISearchLookup',
           'DictionaryTransform')

"""
    pg_fts.fields
    -------------

    Implementation of postgres full text search

    @author: David Miguel

"""

tsvector_re = re.compile(r'[^\w &:\|\!\*\']', flags=re.U)
search_re = re.compile(r'[^\w ]', flags=re.U)


class TSVectorBaseField(Field):

    """
    Base vector field

    :param dictionary: Dictionary name as is in PostgreSQL
        ```pg_catalog.pg_ts_config``` more information in
        :pg_docs:`PostgreSQL documentation 12.6. Dictionaries
        <textsearch-dictionaries.html>`

    :raises: exceptions.FieldError if lookup isn't tsquery, search or isearch

    """
    valid_lookups = ('search', 'isearch', 'tsquery')
    empty_strings_allowed = True

    def __init__(self, dictionary='english', **kwargs):
        """Vector field"""
        kwargs['null'] = True
        kwargs['default'] = ''
        kwargs['editable'] = False
        kwargs['serialize'] = False
        kwargs['db_index'] = False  # use migrations for index creation
        self.dictionary = dictionary
        super(TSVectorBaseField, self).__init__(**kwargs)

    def db_type(self, connection):
        return 'tsvector'

    def get_db_prep_lookup(self, lookup_type, value, connection,
                           prepared=False):

        if lookup_type not in self.valid_lookups:
            raise exceptions.FieldError("'%s' isn't valid Lookup for %s" % (
                lookup_type, self.__class__.__name__))

        return [self.get_db_prep_value(
            self._get_db_prep_lookup(lookup_type, value),
            connection=connection,
            prepared=prepared)]

    @staticmethod
    def _get_db_prep_lookup(lookup_type, value):
        if lookup_type in ('search', 'isearch'):
            operation = ' & ' if lookup_type == 'search' else ' | '
            return "%s" % operation.join(search_re.sub('', value).split())
        elif lookup_type == 'tsquery':
            return "%s" % ' '.join(tsvector_re.sub('', value).split())

    def deconstruct(self):
        name, path, args, kwargs = super(TSVectorBaseField, self).deconstruct()
        path = 'pg_fts.fields.TSVectorBaseField'
        kwargs['dictionary'] = self.dictionary
        return name, path, args, kwargs


class TSVectorField(TSVectorBaseField):
    """
    :param fields: A tuple containing a tuple of fields and rank to be indexed,
        it can be only the field name the default the rank 'D' will be added

        Example::

            ('field_name', ('field_name2', 'A'))

        Will result in::

            (('field_name', 'D'), ('field_name2', 'A'))

    :param dictionary: available options:

        - Can be string with the dictionary name ```pg_catalog.pg_ts_config```
            consult :pg_docs:`PostgreSQL documentation 12.6. Dictionaries
            <textsearch-dictionaries.html>`

        - A TextField name in case of multiple dictionaries

        .. caution::

            Dictionary(ies) used must be installed in your database, check
                ``pg_catalog.pg_ts_config``

    :raises: exceptions.FieldError if lookup isn't tsquery, search or isearch
        or not a valid option dictionary (in case of multiple dictionaries)

    .. caution::

            TSVectorField does not support iexact, it will raise an exception


    """

    DEFAUL_RANK = 'D'
    RANK_LEVELS = ('A', 'B', 'C', 'D')
    default_error_messages = {
        'fields_error': _('Fields must be tuple or list of fields:'),
        'index_error': _('Invalid index:'),
    }

    def __init__(self, fields, dictionary='english', **kwargs):
        self.fields = fields
        super(TSVectorField, self).__init__(dictionary, **kwargs)

    def _get_fields_and_ranks(self):
        for field in self.fields:
            if isinstance(field, (tuple, list)):
                yield (self.model._meta.get_field_by_name(field[0])[0],
                       field[1].upper())
            else:
                yield (self.model._meta.get_field_by_name(field)[0],
                       self.DEFAUL_RANK)

    def check(self, **kwargs):
        errors = super(TSVectorField, self).check(**kwargs)
        for f in self.fields:
            field = None
            if isinstance(f, (tuple, list)):
                if len(f) > 2:
                    errors.append(
                        checks.Error(
                            'Invalid value in fields "%s"' % (str(f),),
                            hint='can be "field" or ("field", "rank")',
                            obj=self,
                            id='fts.E001'
                        )
                    )
                else:
                    field = f[0]
                    if f[1].upper() not in self.RANK_LEVELS:
                        errors.append(
                            checks.Error(
                                'Invalid rank "%s" in "%s"' % (str(f[1]), f),
                                hint=('Available ranks %s' %
                                      ' or '.join(self.RANK_LEVELS)),
                                obj=self,
                                id='fts.E001'
                            )
                        )
            else:
                field = f
            if field and not isinstance(field, six.string_types):
                errors.append(
                    checks.Error(
                        'Invalid value in fields "%s"' % (str(f),),
                        hint='can be name of field or (field, rank)',
                        obj=self,
                        id='fts.E001'
                    )
                )
            elif field:
                try:
                    t = self.model._meta.get_field_by_name(field)[0]
                    if not isinstance(t, (models.CharField, models.TextField)):
                        errors.append(
                            checks.Error(
                                'Field must be CharField or TextField.',
                                hint=None,
                                obj=self,
                                id='fts.E001'
                            )
                        )
                except models.FieldDoesNotExist:
                    errors.append(
                        checks.Error(
                            'FieldDoesNotExistFieldDoesNotExist %s' % str(f),
                            hint=None,
                            obj=self,
                            id='fts.E001'
                        )
                    )

        return errors

    @property
    def description(self):
        return 'TSVector of (%s)' % ', '.join(self.fields)

    def deconstruct(self):
        name, path, args, kwargs = super(TSVectorField, self).deconstruct()
        path = 'pg_fts.fields.TSVectorField'
        kwargs['fields'] = self.fields
        return name, path, args, kwargs

    def get_dictionary(self):
        try:
            df = self.model._meta.get_field(self.dictionary)
            return df.default if df.default else df.options[0][0]
        except models.FieldDoesNotExist:
            return self.dictionary

    def get_transform(self, name):
        transform = super(TSVectorField, self).get_transform(name)
        if transform:
            return transform
        try:
            if name in map(
                    lambda x: x[1],
                    self.model._meta.get_field(self.dictionary).choices):
                return DictionaryTransformFactory(name)
            else:
                raise exceptions.FieldError("The '%s' is not in %s choices" % (
                    name, self.model._meta.get_field(self.dictionary))
                )
        except ValueError:
            pass


class TSVectorTsQueryLookup(Lookup):
    """
    TSVectorField Lookup tsquery

    Raw to_tsquery lookup, check the documentation for more details
        :pg_docs:`12.3.2. Parsing Queries
        <textsearch-controls.html#TEXTSEARCH-PARSING-QUERIES>`

    :param query values: valid PostgreSQL tsquery

        .. caution::
            If the query is not a valid PostgreSQL to_tsquery will result in a
            syntax error

    Example::

        Article.objects.filter(
            tsvector__tsquery="'single-quoted phrases' & prefix:A*B & !not | or | weight:ABC"
        )

    SQL equivalent:

    .. code-block:: sql

        "tsvector" @@ to_tsquery('english', '''singlequoted phrases'' & prefix:A*B & !not | or | weight:ABC')

    .. note::

        The lookup will get dictionary value in
            :class:`~pg_fts.fields.TSVectorField`, this case *english*

        In case of multiple dictionaries see
            :class:`~pg_fts.fields.DictionaryTransform`

    """

    lookup_name = 'tsquery'
    lookup_sql = "%s @@ to_tsquery('%s', %s)"

    def as_sql(self, qn, connection):
        lhs, lhs_params = self.process_lhs(qn, connection)
        rhs, rhs_params = self.process_rhs(qn, connection)
        params = lhs_params + rhs_params
        if hasattr(self.lhs, 'dictionary'):
            dictionary = self.lhs.dictionary
        else:
            dictionary = self.lhs.source.get_dictionary()
        return self.lookup_sql % (lhs, dictionary, rhs), params

    @property
    def output_field(self):
        return TSVectorBaseField(self.dictionary)


TSVectorBaseField.register_lookup(TSVectorTsQueryLookup)


class TSVectorSearchLookup(TSVectorTsQueryLookup):
    """
    TSVectorField Lookup search

    An to_tsquery with AND *&* operator

    :param query values: a string with words

    Example::

        Article.objects.filter(
            tsvector__search="an and query"
        )

    SQL equivalent:::

    .. code-block:: sql

        "tsvector" @@ to_tsquery('english', 'an & and & query')

    .. note::

        The lookup will get dictionary value in
            :class:`~pg_fts.fields.TSVectorField`, this case *english*

        In case of multiple dictionaries see
            :class:`~pg_fts.fields.DictionaryTransform`

    """

    lookup_name = 'search'


TSVectorBaseField.register_lookup(TSVectorSearchLookup)


class TSVectorISearchLookup(TSVectorTsQueryLookup):
    """
    TSVectorField Lookup isearch

    An to_tsquery with OR *|* operator

    :param query values: a string with words

    Example::

        Article.objects.filter(
            tsvector__isearch="an and query"
        )

    SQL equivalent:

    .. code-block:: sql

        "tsvector" @@ to_tsquery('english', 'an | and | query')

    .. note::

        The lookup will get dictionary value in
            :class:`~pg_fts.fields.TSVectorField`, this case *english*

        In case of multiple dictionaries see
            :class:`~pg_fts.fields.DictionaryTransform`

    """

    lookup_name = 'isearch'


TSVectorBaseField.register_lookup(TSVectorISearchLookup)


class DictionaryTransform(Transform):
    """
    TSVectorField dictionary transform

    :param query values: a valid dictionary in
        :class:`~pg_fts.fields.TSVectorField` field dictionary.options

    Example::

        # in models.py

        class Article(models.Model):
            title = models.CharField(max_length=255)
            article = models.TextField()
            dictionary = models.CharField(
                max_length=15,
                choices=(
                    ('english', 'english'),
                    ('portuguese', 'portuguese')
                ),
                default='english',
                db_index=True
            )
            fts_index = TSVectorField(
                (('title', 'A'), 'article'),
                dictionary='dictionary'
            )

    >>> Article.objects.filter(
            tsvector__english__isearch="an and query"
        )
    # will create a ts_query with english dictionary
    # SQL -> to_tsquery('english', 'an | and | query')

    >>> Article.objects.filter(
            tsvector__portuguese__isearch="an and query"
        )
    # will create a ts_query with portuguese dictionary
    # SQL -> to_tsquery('portuguese', 'an | and | query')

    >>> Article.objects.filter(
            tsvector__isearch="an and query"
        )
    # will create a ts_query with english dictionary, it's the default in
    # dictionary, but if there was no default it wold get the 1st option in
    # options
    # SQL -> to_tsquery('english', 'an | and | query')
    # but if dictionary is not in options will raise FieldError
    >>> Article.objects.filter(
            tsvector__japonese__isearch="an and query"  # will raise an error
        )
    FieldError: The 'japonese' is not in article.Article.dictionary choices


    """

    def __init__(self, dictionary, *args,
                 **kwargs):
        super(DictionaryTransform, self).__init__(*args, **kwargs)
        self.dictionary = dictionary

    def as_sql(self, qn, connection):
        lhs, params = qn.compile(self.lhs)
        return "%s" % lhs, params

    @property
    def output_field(self):
        return TSVectorBaseField(self.dictionary)


class DictionaryTransformFactory(object):

    def __init__(self, dictionary):
        self.dictionary = dictionary

    def __call__(self, *args, **kwargs):
        return DictionaryTransform(self.dictionary, *args, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.db.migrations.operations.base import Operation
from pg_fts.fields import TSVectorField


__all__ = ('CreateFTSIndexOperation', 'CreateFTSTriggerOperation',
           'DeleteFTSIndexOperation', 'DeleteFTSTriggerOperation',
           'UpdateVectorOperation')

"""
    pg_fts.migrations
    -----------------

    Migrations module for `pg_fts.fields.TSVectorField`

    @author: David Miguel
"""


class PgFtsSQL(object):
    sql_delete_trigger = ("DROP TRIGGER {model}_{fts_name}_update ON {model};"
                          "DROP FUNCTION {model}_{fts_name}_update()")

    sql_create_trigger = """
CREATE FUNCTION {model}_{fts_name}_update() RETURNS TRIGGER AS $$
BEGIN
    IF TG_OP = 'INSERT' THEN
        new.{fts_name} = {vectors};
    END IF;
    IF TG_OP = 'UPDATE' THEN
        IF {fts_fields} THEN
            new.{fts_name} = {vectors};
        END IF;
    END IF;
RETURN NEW;
END;
$$ LANGUAGE 'plpgsql';
CREATE TRIGGER {model}_{fts_name}_update BEFORE INSERT OR UPDATE ON {model}
FOR EACH ROW EXECUTE PROCEDURE {model}_{fts_name}_update()"""

    sql_create_index = ("CREATE INDEX {model}_{fts_name} ON {model} "
                        "USING {fts_index}({fts_name})")

    sql_delete_index = 'DROP INDEX {model}_{fts_name}'

    sql_update_vector = 'UPDATE {model} SET {vector} = {fields}'

    def delete_trigger(self, model, field):
        return self.sql_delete_trigger.format(
            model=model._meta.db_table,
            fts_name=field.get_attname_column()[1]
        )

    def create_fts_trigger(self, model, vector_field):

        fields = []
        vectors = []

        if not isinstance(vector_field, TSVectorField):
            raise AttributeError

        try:
            dict_field = model._meta.get_field_by_name(vector_field.dictionary)[0]
            dictionary = "NEW.%s::regconfig" % (
                dict_field.get_attname_column()[1])
            fields.append('NEW.{0} <> OLD.{0}'.format(vector_field.dictionary))
        except:
            dictionary = "'%s'" % vector_field.dictionary

        for field, rank in vector_field._get_fields_and_ranks():
            fields.append('NEW.{0} <> OLD.{0}'.format(
                field.get_attname_column()[1]))
            vectors.append(self._get_vector_for_field(field, rank, dictionary))

        return self.sql_create_trigger.format(
            model=model._meta.db_table,
            fts_name=vector_field.get_attname_column()[1],
            fts_fields=' OR '.join(fields),
            vectors=' || '.join(vectors)
        )

    def update_vector(self, model, vector_field):
        vectors = []
        sql_fn = "setweight(to_tsvector(%s, COALESCE(%s, '')), '%s')"

        if not isinstance(vector_field, TSVectorField):
            raise AttributeError

        try:
            dict_field = model._meta.get_field_by_name(vector_field.dictionary)[0]
            dictionary = "%s::regconfig" % (
                dict_field.get_attname_column()[1])
        except:
            dictionary = "'%s'" % vector_field.dictionary

        for field, rank in vector_field._get_fields_and_ranks():
            vectors.append(sql_fn % (
                dictionary, field.get_attname_column()[1], rank))

        return self.sql_update_vector.format(
            model=model._meta.db_table,
            vector=vector_field.get_attname_column()[1],
            fields=' || '.join(vectors)
        )

    def _get_vector_for_field(self, field, weight, dictionary):
        return "setweight(to_tsvector(%s, COALESCE(NEW.%s, '')), '%s')" % (
            dictionary, field.get_attname_column()[1], weight
        )

    def create_index(self, model, vector_field, index):
        return self.sql_create_index.format(
            model=model._meta.db_table,
            fts_index=index,
            fts_name=vector_field.get_attname_column()[1]
        )

    def delete_index(self, model, vector_field):
        return self.sql_delete_index.format(
            model=model._meta.db_table,
            fts_name=vector_field.get_attname_column()[1]
        )


class BaseVectorOperation(Operation):
    """
    Base migrations class

    :param name: The Model name

    :param fts_vector: The :class:`~pg_fts.fields.TSVectorField` field name
    """

    reduces_to_sql = True
    reversible = True
    sql_creator = PgFtsSQL()
    forward_fn = None
    backward_fn = None

    def __init__(self, name, fts_vector):
        self.name = name
        self.fts_vector = fts_vector

    def state_forwards(self, app_label, state):
        pass

    def database_forwards(self, app_label, schema_editor, from_state,
                          to_state):

        model = from_state.render().get_model(app_label, self.name)
        vector_field = model._meta.get_field_by_name(self.fts_vector)[0]
        schema_editor.execute(self.forward_fn(
            model,
            vector_field
        ))

    def database_backwards(self, app_label, schema_editor, from_state,
                           to_state):

        model = from_state.render().get_model(app_label, self.name)
        vector_field = model._meta.get_field_by_name(self.fts_vector)[0]

        schema_editor.execute(self.backward_fn(
            model,
            vector_field
        ))

    def describe(self):
        return "Create trigger `%s` for model `%s`" % (
            self.fts_vector, self.name
        )


class UpdateVectorOperation(BaseVectorOperation):
    """
    Updates changes to :class:`~pg_fts.fields.TSVectorField` for existing
    models

    :param name: The Model name

    :param fts_vector: The :class:`~pg_fts.fields.TSVectorField` field name
    """

    def __init__(self, name, fts_vector):
        self.name = name
        self.fts_vector = fts_vector
        self.forward_fn = self.sql_creator.update_vector

    def database_backwards(self, app_label, schema_editor, from_state,
                           to_state):
        pass

    def describe(self):
        return "Create trigger `%s` for model `%s`" % (
            self.fts_vector, self.name
        )


class CreateFTSTriggerOperation(BaseVectorOperation):
    """
    Creates a :pg_docs:`custom trigger <textsearch-features.html#TEXTSEARCH-UPDATE-TRIGGERS>`
    for updating the :class:`~pg_fts.fields.TSVectorField` with rank values

    :param name: The Model name

    :param fts_vector: The :class:`~pg_fts.fields.TSVectorField` field name
    """

    def __init__(self, name, fts_vector):
        self.name = name
        self.fts_vector = fts_vector
        self.forward_fn = self.sql_creator.create_fts_trigger
        self.backward_fn = self.sql_creator.delete_trigger

    def database_backwards(self, app_label, schema_editor, from_state,
                           to_state):

        model = from_state.render().get_model(app_label, self.name)
        vector_field = model._meta.get_field_by_name(self.fts_vector)[0]

        schema_editor.execute(self.backward_fn(
            model,
            vector_field
        ))

    def describe(self):
        return "Create trigger `%s` for model `%s`" % (
            self.fts_vector, self.name
        )


class DeleteFTSTriggerOperation(BaseVectorOperation):
    """
    Deletes trigger generated by :class:`~pg_fts.migrations.CreateFTSTriggerOperation`

    :param name: The Model name

    :param fts_vector: The :class:`~pg_fts.fields.TSVectorField` field name
    """

    def __init__(self, name, fts_vector):
        self.name = name
        self.fts_vector = fts_vector
        self.forward_fn = self.sql_creator.delete_trigger
        self.backward_fn = self.sql_creator.create_fts_trigger

    def describe(self):
        return "Delete trigger `%s` for model `%s`" % (
            self.fts_vector, self.name
        )


class CreateFTSIndexOperation(BaseVectorOperation):
    """
    Creates a index for :class:`~pg_fts.fields.TSVectorField`

    :param name: The Model name
    :param fts_vector: The :class:`~pg_fts.fields.TSVectorField` field name
    :param index: The type of index 'gin' or 'gist' for more information go to
        :pg_docs:`PostgreSQL documentation 12.9. GiST and GIN Index Types
        <textsearch-indexes.html>`
    """

    # http://www.postgresql.org/docs/9.3/static/textsearch-indexes.html
    INDEXS = ('gin', 'gist')

    def __init__(self, name, fts_vector, index):
        assert index in self.INDEXS, "Invalid index '%s'. Options %s " % (
            index, ', '.join(self.INDEXS))
        self.name = name
        self.fts_vector = fts_vector
        self.index = index

    def state_forwards(self, app_label, state):
        pass

    def database_forwards(self, app_label, schema_editor, from_state,
                          to_state):

        model = from_state.render().get_model(app_label, self.name)
        vector_field = model._meta.get_field_by_name(self.fts_vector)[0]
        if not isinstance(vector_field, TSVectorField):
            raise AttributeError
        schema_editor.execute(self.sql_creator.create_index(
            model, vector_field, self.index
        ))

    def database_backwards(self, app_label, schema_editor, from_state,
                           to_state):

        model = from_state.render().get_model(app_label, self.name)
        vector_field = model._meta.get_field_by_name(self.fts_vector)[0]

        schema_editor.execute(self.sql_creator.delete_index(
            model,
            vector_field
        ))

    def describe(self):
        return "Create %s index `%s` for model `%s`" % (
            self.index, self.fts_vector, self.name
        )


class DeleteFTSIndexOperation(CreateFTSIndexOperation):
    """
    Removes index created by :class:`~pg_fts.migrations.CreateFTSIndexOperation`

    :param name: The Model name
    :param fts_vector: The :class:`~pg_fts.fields.TSVectorField` field name
    :param index: The type of index 'gin' or 'gist' for more information go to
    """

    def database_forwards(self, app_label, schema_editor, from_state,
                          to_state):
        model = from_state.render().get_model(app_label, self.name)
        vector_field = model._meta.get_field_by_name(self.fts_vector)[0]

        schema_editor.execute(self.sql_creator.delete_index(
            model,
            vector_field
        ))

    def database_backwards(self, app_label, schema_editor, from_state,
                           to_state):

        model = from_state.render().get_model(app_label, self.name)
        vector_field = model._meta.get_field_by_name(self.fts_vector)[0]
        if not isinstance(vector_field, TSVectorField):
            raise AttributeError
        schema_editor.execute(self.sql_creator.create_index(
            model, vector_field, self.index))

    def describe(self):
        return "Delete %s index `%s` for model `%s`" % (
            self.index, self.fts_vector, self.name
        )



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name='django-pg_fts',
    version='0.1.1',
    packages=['pg_fts'],
    include_package_data=True,
    license='BSD License',
    description='Implementation of PostgreSQL Full Text Search for django 1.7',
    long_description=README,
    url='http://www.dvdmgl.com/',
    author='dvdmgl',
    author_email='dvdmgl@gmail.com',
    requires=[
        'Django (>=1.7)',
        'psycopg2'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    keywords='django postgres postgresql pgsql full text search fts',
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.test import TestCase
from testapp.models import TSQueryModel, TSMultidicModel, Related
from django.core import exceptions
from django.utils import encoding
from pg_fts.aggregates import FTSRank

__all__ = ('TestQueryingSingleDictionary', 'TestQueryingMultiDictionary')


class TestQueryingSingleDictionary(TestCase):

    def setUp(self):
        a = TSQueryModel.objects.create(
            title='para for os the mesmo same malucos crazy',
            body="""para for os the mesmo same malucos crazy que that tomorow
salvão save o the planeta planet"""
        )

        b = TSQueryModel.objects.create(
            title='malucos crazy como like eu me',
            body="""para for os the mesmo same malucos crazy que that tomorow
salvão save o the planeta planet"""
        )
        Related.objects.create(single=a)
        Related.objects.create(single=b)

    def test_search(self):
        q = TSQueryModel.objects.filter(tsvector__search='para mesmo')
        self.assertEqual(len(q), 2)
        self.assertIn('''WHERE "testapp_tsquerymodel"."tsvector" @@ to_tsquery('english', para & mesmo)''',
                      str(q.query))
        self.assertEqual(
            q[0].title, 'para for os the mesmo same malucos crazy')
        self.assertEqual(
            len(TSQueryModel.objects.filter(
                tsvector__search='para mesmo todos')),
            0
        )

    def test_isearch(self):
        q = TSQueryModel.objects.filter(tsvector__isearch='para mesmo')
        self.assertEqual(
            q[0].title, 'para for os the mesmo same malucos crazy')

        self.assertIn('''WHERE "testapp_tsquerymodel"."tsvector" @@ to_tsquery('english', para | mesmo)''',
                      str(q.query))
        self.assertEqual(
            len(TSQueryModel.objects.filter(
                tsvector__isearch='para mesmo todos')),
            2
        )

    def test_tsquery(self):
        q = TSQueryModel.objects.filter(tsvector__tsquery='para & mesmo')
        self.assertEqual(
            q[0].title, 'para for os the mesmo same malucos crazy')
        self.assertEqual(
            len(TSQueryModel.objects.filter(
                tsvector__tsquery='para | mesmo | todos')),
            2
        )
        self.assertIn('''WHERE "testapp_tsquerymodel"."tsvector" @@ to_tsquery('english', para & mesmo)''',
                      str(q.query))

    def test_tsquery_re(self):
        q = TSQueryModel.objects.filter(
            tsvector__tsquery="'single-quoted phrases' & prefix:A*B & !not  | or | weight:ABC"
        )
        self.assertEqual(len(q), 0)  # just test for no errors

    def test_nonasc(self):  # fail
        ao = TSQueryModel.objects.filter(
            tsvector__isearch='canção & é & vèz',
        )
        self.assertIn(
            b'''WHERE "testapp_tsquerymodel"."tsvector" @@ to_tsquery('english', can\xc3\xa7\xc3\xa3o | \xc3\xa9 | v\xc3\xa8z)''',
            encoding.smart_bytes(ao.query)
        )

    def test_related_search(self):
        q = Related.objects.filter(single__tsvector__search='para mesmo')
        self.assertEqual(len(q), 2)



class TestQueryingMultiDictionary(TestCase):

    def setUp(self):
        title = 'para for os the mesmo same malucos crazy'
        body = """para for os the mesmo same malucos crazy que that tomorow
salvão save o the planeta planet"""
        pt = TSMultidicModel.objects.create(
            title=title,
            body=body,
            dictionary='english'
        )

        en = TSMultidicModel.objects.create(
            title=title,
            body=body,
            dictionary='portuguese'
        )

        Related.objects.create(
            multiple=pt
        )

        Related.objects.create(
            multiple=en
        )

    def test_different_vectors_for_different_dictionaries(self):
        pt = TSMultidicModel.objects.filter(dictionary='portuguese')
        en = TSMultidicModel.objects.filter(dictionary='english')
        self.assertNotEqual(pt[0].tsvector, en[0].tsvector)

    def test_dictinary_transform_search(self):
        # `para``os` are stopwords in portuguese
        self.assertEqual(len(
            TSMultidicModel.objects.filter(
                tsvector__portuguese__tsquery='para & os',
                dictionary='portuguese'
            )
            ), 0)

        self.assertEqual(len(
            TSMultidicModel.objects.filter(
                tsvector__portuguese__search='para os',
                dictionary='portuguese'
            )
            ), 0)

        self.assertEqual(len(
            TSMultidicModel.objects.filter(
                tsvector__portuguese__isearch='para os',
                dictionary='portuguese'
            )
            ), 0)

        self.assertEqual(len(
            TSMultidicModel.objects.filter(
                tsvector__english__tsquery='para & os',
                dictionary='english'
            )
            ), 1)

        self.assertEqual(len(
            TSMultidicModel.objects.filter(
                tsvector__english__search='para os',
                dictionary='english'
            )
            ), 1)

        self.assertEqual(len(
            TSMultidicModel.objects.filter(
                tsvector__english__isearch='para & os',
                dictionary='english'
            )
            ), 1)

    def test_related_multidict(self):
        self.assertEqual(len(
            Related.objects.filter(
                multiple__tsvector__portuguese__tsquery='para & os',
                multiple__dictionary='portuguese'
            )
            ), 0)

        self.assertEqual(len(
            Related.objects.filter(
                multiple__tsvector__portuguese__search='para os',
                multiple__dictionary='portuguese'
            )
            ), 0)

        self.assertEqual(len(
            Related.objects.filter(
                multiple__tsvector__portuguese__isearch='para os',
                multiple__dictionary='portuguese'
            )
            ), 0)

        self.assertEqual(len(
            Related.objects.filter(
                multiple__tsvector__english__tsquery='para & os',
                multiple__dictionary='english'
            )
            ), 1)

        self.assertEqual(len(
            Related.objects.filter(
                multiple__tsvector__english__search='para os',
                multiple__dictionary='english'
            )
            ), 1)

        self.assertEqual(len(
            Related.objects.filter(
                multiple__tsvector__english__isearch='para & os',
                multiple__dictionary='english'
            )
            ), 1)

    def test_transform_dictionary_exception(self):
        with self.assertRaises(exceptions.FieldError) as msg:
            TSMultidicModel.objects.filter(tsvector__nodict='malucos'),
        self.assertEqual(str(msg.exception), "The 'nodict' is not in testapp.TSMultidicModel.dictionary choices")

    def test_transform_exception(self):
        with self.assertRaises(exceptions.FieldError) as msg:
            list(TSMultidicModel.objects.filter(tsvector__portuguese='malucos'))
        self.assertEqual(str(msg.exception), "'exact' isn't valid Lookup for TSVectorBaseField")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.utils.encoding import python_2_unicode_compatible

from pg_fts.fields import TSVectorField
from django.db import models


@python_2_unicode_compatible
class TSQueryModel(models.Model):
    title = models.CharField(max_length=50)
    body = models.TextField()

    tsvector = TSVectorField(('title', 'body'))

    def __str__(self):
        return self.title


@python_2_unicode_compatible
class TSMultidicModel(models.Model):
    title = models.CharField(max_length=50)
    body = models.TextField()
    dictionary = models.CharField(
        max_length=15,
        choices=(('english', 'english'), ('portuguese', 'portuguese')),
        default='english'
    )
    tsvector = TSVectorField((('title', 'A'), 'body'),
                             dictionary='dictionary')

    def __str__(self):
        return self.title


class Related(models.Model):
    single = models.ForeignKey(TSQueryModel, blank=True, null=True)
    multiple = models.ForeignKey(TSMultidicModel, blank=True, null=True)
